from zs_utils.api.ozon.base_api import OzonAPI


class OzonGetCategoriesTreeAPI(OzonAPI):
    """
    Docs: https://docs.ozon.ru/api/seller/#operation/CategoryAPI_GetCategoryTree
    """

    resource_method = "v2/category/tree"
    allowed_params = ["category_id", "language"]


class OzonGetCategoryAttributeAPIv3(OzonAPI):
    """
    Docs: https://docs.ozon.ru/api/seller/#operation/CategoryAPI_GetCategoryAttributesV3
    """

    resource_method = "v3/category/attribute"
    required_params = ["category_id"]
    allowed_params = ["language", "attribute_type"]


class OzonGetCategoryAttributeDictionaryAPI(OzonAPI):
    """
    Docs: https://docs.ozon.ru/api/seller/#operation/CategoryAPI_DictionaryValueBatch
    """

    resource_method = "v2/category/attribute/values"
    required_params = ["attribute_id", "category_id", "limit"]
    allowed_params = ["language", "last_value_id"]



from model_utils.models import TimeStampedModel, UUIDModel
from django.db import models

from zs_utils.file_utils import get_support_file_upload_path
from zs_utils.support import constants


__all__ = [
    "AbstractSupportTicket",
    "AbstractSupportTicketMessage",
    "AbstractSupportTicketMessageFile",
]


class AbstractSupportTicket(TimeStampedModel, UUIDModel):
    """
    Данные тикета (запросов в поддержку)
    """
    number = models.TextField(verbose_name="Номер обращения")
    subject = models.CharField(max_length=300, verbose_name="Тема")
    status = models.TextField(choices=constants.SUPPORT_TICKET_STATUSES, verbose_name="Статус")
    question_type = models.TextField(verbose_name="Тип вопроса", choices=constants.SUPPORT_TICKET_QUESTION_TYPES)

    # Нужно определять в наследниках
    user = None  # models.ForeignKey(User, on_delete=models.CASCADE, related_name="tickets", verbose_name="Создатель")
    manager = None  # models.ForeignKey(User, null=True, on_delete=models.SET_NULL, verbose_name="Менеджер")

    class Meta:
        abstract = True
        ordering = ["-created"]

    @property
    def client_status(self) -> constants.SUPPORT_TICKET_CLIENT_STATUSES:
        if self.status in constants.SUPPORT_TICKET_ACTIVE_STATUSES:
            if self.manager:
                last_message = self.last_message
                if last_message and (last_message.sender == self.manager):
                    return constants.SUPPORT_TICKET_CLIENT_STATUSES.RESPONDED
            return constants.SUPPORT_TICKET_CLIENT_STATUSES.PENDING
        else:
            return constants.SUPPORT_TICKET_CLIENT_STATUSES.CLOSED

    @property
    def is_active(self):
        """
        Тикет активный
        """
        return self.status in constants.SUPPORT_TICKET_ACTIVE_STATUSES

    @property
    def unread_messages(self) -> dict:
        """
        Не прочтённые сообщения
        """
        unread = self.messages.filter(is_system=False, is_viewed=False)
        return {
            "user": unread.exclude(sender=self.user).count(),
            "manager": unread.filter(sender=self.user).count(),
        }

    @property
    def last_message(self):
        """
        Последнее сообщение в данном обращении
        """
        return self.messages.filter(is_system=False).order_by("-created").first()

    @property
    def all_users(self) -> set:
        """
        Получение всех пользователей, которые участвуют в обращении
        """
        res = list(self.messages.values_list("sender", flat=True).distinct())
        if self.user_id not in res:
            res.append(self.user_id)
        if self.manager_id and (self.manager_id not in res):
            res.append(self.manager_id)
        return res


class AbstractSupportTicketMessage(TimeStampedModel, UUIDModel):
    """
    Данные сообщения тикета
    """
    text = models.TextField(max_length=1500, null=True, verbose_name="Сообщение")
    is_system = models.BooleanField(verbose_name="Системное сообщение")
    is_viewed = models.BooleanField(default=False, verbose_name="Просмотрено получателем")

    # Нужно определять в наследниках
    ticket = None  # models.ForeignKey("SupportTicket", on_delete=models.CASCADE, related_name="messages", related_query_name="message", verbose_name="Тикет")
    sender = None  # models.ForeignKey(to=User, blank=True, null=True, on_delete=models.SET_NULL, verbose_name="Автор сообщения")

    class Meta:
        abstract = True
        ordering = ["-created"]

    @property
    def is_from_manager(self) -> bool:
        """
        Сообщение от менеджера
        """
        return self.ticket.manager and (self.sender == self.ticket.manager)

    @property
    def is_from_client(self) -> bool:
        """
        Сообщение от пользователя
        """
        return self.sender == self.ticket.user

    @property
    def recipient(self):
        """
        Получатель сообщения
        """
        if self.sender == self.ticket.user:
            return self.ticket.manager
        else:
            return self.ticket.user


class AbstractSupportTicketMessageFile(UUIDModel):
    """
    Данные прикреплённого файла к сообщению тикета
    """
    file = models.FileField(upload_to=get_support_file_upload_path, verbose_name="Приложенный файл")

    # Нужно определять в наследниках
    user = None  # models.ForeignKey(User, on_delete=models.CASCADE, related_name="+", verbose_name="Пользователь")
    ticket_message = None  # models.ForeignKey("SupportTicketMessage", null=True, on_delete=models.CASCADE, related_name="files", related_query_name="file", verbose_name="Сообщение")

    @property
    def is_used(self) -> bool:
        if not self.ticket_message:
            raise NotImplemented
        return bool(self.ticket_message_id)

    @property
    def name(self) -> str:
        return self.file.name.split("/")[-1]

    class Meta:
        abstract = True



from model_utils import Choices

from django.conf import settings
from django.utils.translation import gettext_lazy as _


SUPPORT_TICKET_QUESTION_TYPES = getattr(settings, 'SUPPORT_TICKET_QUESTION_TYPES')

SUPPORT_TICKET_STATUSES = Choices(
    ("PENDING", _("Ожидает рассмотрения")),
    ("OPEN", _("Открыто")),
    ("CLOSED_BY_USER", _("Закрыто пользователем")),
    ("CLOSED_BY_MANAGER", _("Закрыто менеджером")),
    ("CLOSED_AUTO", _("Закрыто из-за отсутствия активности")),
)

SUPPORT_TICKET_ACTIVE_STATUSES = SUPPORT_TICKET_STATUSES.subset("PENDING", "OPEN")

SUPPORT_TICKET_ACTIVE_STATUSES_LIST = [item[0] for item in SUPPORT_TICKET_ACTIVE_STATUSES]

SUPPORT_TICKET_CLIENT_STATUSES = Choices(
    ("PENDING", _("В работе")),
    ("RESPONDED", _("Получен ответ")),
    ("CLOSED", _("Закрыто")),
)

MAX_OPEN_TICKET = getattr(settings, 'MAX_OPEN_TICKET', 10)



from django.dispatch import receiver

from zs_utils.support import models, signals, constants, services


@receiver(signal=signals.support_ticket_status_changed)
def create_system_message(ticket: models.AbstractSupportTicket, prev_status: constants.SUPPORT_TICKET_STATUSES, **kwargs):
    services.CommonSupportTicketService.create_system_message_after_status_change(
        ticket=ticket,
        prev_status=prev_status,
    )



from django.db import transaction
from rest_framework.decorators import action
from rest_framework.parsers import MultiPartParser

from zs_utils.views import CustomModelViewSet


__all__ = [
    "BaseSupportTicketView",
    "BaseSupportTicketMessageView",
    "BaseSupportTicketMessageFileView",
]


class BaseSupportTicketView(CustomModelViewSet):
    """
    Абстрактный View для создания и просмотра тикетов (SupportTicket)
    """
    not_allowed_actions = [
        "destroy",
        "update",
        "partial_update",
    ]
    staff_permission_classes = []

    @classmethod
    def get_support_ticket_service_model(cls):
        raise NotImplementedError("В сервисе должен быть определён метод получения модели SupportTicketService")

    @classmethod
    def get_create_support_ticket_serializer_model(cls):
        raise NotImplementedError(
            "В сервисе должен быть определён метод получения модели CreateSupportTicketSerializer"
        )

    def get_queryset_filter_kwargs(self) -> dict:
        user = self.get_user()
        if user.is_staff:
            return {}
        else:
            return {"user": user}

    @action(detail=True, methods=["POST"])
    def close(self, request, *args, **kwargs):
        """
        Закрытие тикета
        """
        self.get_support_ticket_service_model().close_ticket(user=self.get_user(), ticket=self.get_object())
        return self.build_response()

    def create(self, request, *args, **kwargs):
        data = self.get_validated_data(serializer_class=self.get_create_support_ticket_serializer_model())
        message = data.pop("message")
        user = self.get_user()
        with transaction.atomic():
            instance = self.get_support_ticket_service_model().create_ticket(user=user, **data)
            self.get_support_ticket_service_model().create_ticket_message(
                ticket=instance, sender=user, is_system=False, **message
            )
        return self.build_response(data=self.serializer_class(instance).data)

    @action(detail=True, methods=["POST"], permission_classes=staff_permission_classes)
    def take_to_work(self, request, *args, **kwargs):
        """
        Взятие тикета в работу (для staff пользователей)
        """
        if not self.staff_permission_classes:
            raise NotImplementedError("В сервисе должен быть определён staff_permission_classes")

        self.get_support_ticket_service_model().take_ticket(manager=self.get_user(), ticket=self.get_object())
        return self.build_response()

    @action(detail=True, methods=["POST"])
    def reopen(self, request, *args, **kwargs):
        """
        Открыть закрытый тикет
        """
        self.get_support_ticket_service_model().reopen_ticket(user=self.get_user(), ticket=self.get_object())
        return self.build_response()

    @action(detail=True, methods=["POST"])
    def set_viewed(self, request, *args, **kwargs):
        self.get_support_ticket_service_model().set_ticket_viewed(user=self.get_user(), ticket=self.get_object())
        return self.build_response()


class BaseSupportTicketMessageView(CustomModelViewSet):
    """
    Абстрактный View для создания/удаления/обновления/просмотра сообщений тикета (SupportTicketMessage)
    """
    not_allowed_actions = [
        "update",
        "partial_update",
        "destroy",
    ]


class BaseSupportTicketMessageFileView(CustomModelViewSet):
    """
    Абстрактный View для создания/удаления/обновления/просмотра файлов сообщений (SupportTicketMessageFile)
    """
    parser_classes = (MultiPartParser,)
    not_allowed_actions = [
        "update",
        "partial_update",
        "destroy",
    ]



from django.apps import AppConfig


class SupportConfig(AppConfig):
    name = "zs_utils.support"
    label = "support"

    def ready(self) -> None:
        import zs_utils.support.signals  # noqa
        import zs_utils.support.receivers  # noqa
